//-----------------------------------------------------------------------------
// linux/drivers/usb/misc/epson-13u11/s1d13u11_ctl.h  
//
// This file is the header for the Epson S1D13U11 display controls.
// 
// Copyright(C) SEIKO EPSON CORPORATION 2010-2013. All rights reserved.
//
// This driver software is distributed as is, without any warranty of any kind,
// either express or implied as further specified in the GNU Public License. This
// software may be used and distributed according to the terms of the GNU Public
// License, version 2. See the file COPYING in the main directory of this archive
// for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//
//---------------------------------------------------------------------------- 

#ifndef __S1D13U11_CTL_H__
#define	__S1D13U11_CTL_H__


// S1D13U11_LCDC_CMD defines
#define	S1D13U11_LCDC_CMD_ACCESS		0


// S1D13U11_DISP_CMD defines
#define	S1D13U11_DISP_CMD_CFG_GETINFO				0xFD
#define	S1D13U11_DISP_CMD_CFG_DOWNLOAD				0xFE
#define	S1D13U11_DISP_CMD_CFG_SWITCH				0xFF
#define	S1D13U11_DISP_CMD_LCDC_READ					0x00
#define	S1D13U11_DISP_CMD_LCDC_WRITE				0x02
#define	S1D13U11_DISP_CMD_LCDC_VRAM_ACC_ENABLE		0x04
#define	S1D13U11_DISP_CMD_LCDC_VRAM_ACC_DISABLE		0x05
#define	S1D13U11_DISP_CMD_LCDC_WAKEUP_ON_CONFIG		0x06
#define	S1D13U11_DISP_CMD_I2C_CONFIG				0x20
#define	S1D13U11_DISP_CMD_I2C_ACCESS				0x21
#define	S1D13U11_DISP_CMD_SPI_CONFIG				0x40
#define	S1D13U11_DISP_CMD_SPI_ACCESS				0x41
#define	S1D13U11_DISP_CMD_SPI_SEQUENCE_START		0x43
#define	S1D13U11_DISP_CMD_SPI_SEQUENCE_STOP			0x44
#define	S1D13U11_DISP_CMD_GPIO_CONFIG				0x80
#define	S1D13U11_DISP_CMD_GPIO_INT_CONFIG			0x81
#define	S1D13U11_DISP_CMD_GPIO_INT_CONTROL			0x82
#define	S1D13U11_DISP_CMD_GPIO_READ					0x83
#define	S1D13U11_DISP_CMD_GPIO_WRITE				0x84
#define	S1D13U11_DISP_CMD_KEYSCAN_CONTROL			0x90
#define	S1D13U11_DISP_CMD_KEYSCAN_READ				0x91
#define	S1D13U11_DISP_CMD_BUZZER_CONTROL			0xB0
#define	S1D13U11_DISP_CMD_EVENT_INT_CONTROL			0xC0


// S1D13U11_LCDC_CTL defines
#define	S1D13U11_LCDC_CTL_WRITE			0
#define	S1D13U11_LCDC_CTL_READ			1
#define	S1D13U11_LCDC_CTL_BUS_8			8
#define	S1D13U11_LCDC_CTL_BUS_16		16
#define S1D13U11_LCDC_CTL_CMD			32	// Application Command Use

#define	S1D13U11_LCDC_CTL_TESTCMD		50	// Test Application Command Use
#define	S1D13U11_LCDC_CTL_TESTDATA		51	// Test Application Picture Use
#define	S1D13U11_LCDC_CTL_DISPALLOC		52	// Test Display Picture Allocate
#define	S1D13U11_LCDC_CTL_DISPFREE		53	// Test Display Picture Buffer Free
#define	S1D13U11_LCDC_CTL_RESETREC		54	// Reset Recovery Send
#define	S1D13U11_LCDC_CTL_SOFTRESET		55	// Test Application Soft Reset
#define	S1D13U11_LCDC_CTL_BUSRESET		56	// Test Application USB Bus Reset


// S1D13U11_FB_CMD defines
//   commented defines are not currently implemented in ioctls
//#define S1D13U11_FB_CMD_GET_VSCREENINFO		0x01
//#define S1D13U11_FB_CMD_GET_FSCREENINFO		0x02
//#define S1D13U11_FB_CMD_PUT_SCREEN			0x03	// use application
//#define S1D13U11_FB_CMD_CYCLE_UPDATESCREEN	0x04	// use application
#define	S1D13U11_FB_CMD_LCDC_READ			0x11	// read a register
#define	S1D13U11_FB_CMD_LCDC_WRITE			0x12	// write to the registers
//#define	S1D13U11_FB_CMD_RECT_SCREEN			0x13	// use application
//#define	S1D13U11_FB_DATA_PIP_WRITE			0x14	// use application
//#define	S1D13U11_FB_CMD_PIP_SET_POS			0x15	// use application
//#define	S1D13U11_FB_CMD_PIP_NON_DISP		0x16	// use application
//#define	S1D13U11_FB_CMD_CYCLE_START			0x17	// use application
//#define	S1D13U11_FB_CMD_CYCLE_STOP			0x18	// use application
#define S1D13U11_FB_CMD_FLUSH_VFB			0x19	//flush the virtual frame buffer


// S1D13U11_LCDC_RESP defines
#define S1D13U11_LCDC_RESP_SUSPEND			0x01	// suspend
#define S1D13U11_LCDC_RESP_RESUME			0x02	// resume
#define S1D13U11_LCDC_RESP_STATUS			0x03	// get power status


// S1D13U11_LCDC_STATUS defines
#define	S1D13U11_LCDC_STATUS_SUSPEND		0x00	// Device Suspend
#define	S1D13U11_LCDC_STATUS_RUN			0x01	// Device Running


// S1D13U11 LCDC control structure
struct s1d13u11_ctl {
	int				respcode ;		// response code
	unsigned char	acc;			// access width
	unsigned long	addr;			// address
	unsigned int	size;			// length
	unsigned char	*buf;			// Data
	unsigned char	value;			// reg return value
	unsigned int	status_size ;	// Status size
	unsigned char	*status_buf ;	// Status data
	unsigned int	allocsize ;		// FB allocate size (test tool use)
};


// S1D13U11 LCDC soft reset structure
struct s1d13u11_softreset {
	unsigned char	bmRequestType;
	unsigned char	bRequest;
	unsigned char	wValue;
	unsigned char	wIndex;
	unsigned char	wLength;
	unsigned char	*Data;
} ;


// S1D13U11 LCDC frame buffer rectangle structure
struct s1d13u11_rect  {
	int				xpos ;		// Rect X Start Position
	int				ypos ;		// Rect Y Start Position
	unsigned int	width ;		// Rect Width
	unsigned int	height ;	// Rect Height
	unsigned char	*buf ;		// Rect image data
} ;


// S1D13U11 LCDC PIP rectangle structure
struct s1d13u11_pip  {
	int				xpos ;			// PIP Window X Start Position
	int				ypos ;			// PIP Window Y Start Position
	unsigned int	width ;			// PIP Window Width
	unsigned int	height ;		// PIP Window Height
	unsigned int	pictWidth ;		// PIP Picture Width
	unsigned int	pictHeight ;	// PIP Picture Height
	unsigned int	size ;			// PIP Picture Size
	unsigned char	*data;			// PIP Picture Data
} ;


// S1D13U11 LCDC PIP position structure
struct s1d13u11_pip_pos  {
	int		xpos ;			// PIP X Start Position
	int		ypos ;			// PIP Y Start Position
} ;


#endif	/* __S1D13U11_CTL_H__ */
