//-----------------------------------------------------------------------------
// linux/drivers/usb/misc/epson-13u11/s1d13u11.h  
//
// This file is the header for the Epson S1D13U11 core driver.
// 
// Copyright(C) SEIKO EPSON CORPORATION 2010-2013. All rights reserved.
//
// This driver software is distributed as is, without any warranty of any kind,
// either express or implied as further specified in the GNU Public License. This
// software may be used and distributed according to the terms of the GNU Public
// License, version 2. See the file COPYING in the main directory of this archive
// for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//
//---------------------------------------------------------------------------- 

#ifndef __S1D13U11_H__
#define	__S1D13U11_H__

#include <linux/poll.h>

#include "s1d13u11_display.h"
#include "s1d13u11_ctl.h"


#ifdef CONFIG_S1D13U11_DEBUG_MESSAGES
#define	DPRINT(args...)		printk(args)
#else
#define	DPRINT(args...)		
#endif

#ifdef PG_RELIEF
#define	S1D13U11LCDC_LOCK		
#define	S1D13U11LCDC_UNLOCK		
#define	UCHARIS(x)		(unsigned char)((x)&0xFF)
#else
#define	S1D13U11LCDC_LOCK		unsigned long iflag; spin_lock_irqsave(&lcdc_lock, iflag)
#define	S1D13U11LCDC_UNLOCK		spin_unlock_irqrestore(&lcdc_lock, iflag)

#define	S1D13U11_CMDLOCK		down_interruptible(&lcdc_cmdlock)
#define	S1D13U11_CMDUNLOCK		up(&lcdc_cmdlock)

#define	UCHARIS(x)		(x)
#endif

// S1D13U11_NAME defines 
#define	S1D13U11_NAME_DRIVER			"s1d13u11"

// S1D13U11_SEL defines
#define	S1D13U11_SEL_NOT			0
#define	S1D13U11_SEL_YES			1

// S1D13U11_NODE defines
#define	S1D13U11_NODE_FB_MAJOR		29	// Frame buffer
#define	S1D13U11_NODE_FB_MINOR		1	// UNDECIDED
#define	S1D13U11_NODE_TS_MAJOR		10	// Touch screen
#define	S1D13U11_NODE_TS_MINOR		240
#define S1D13U11_NODE_INT_MAJOR		180	// Protocol Interrupt
#define S1D13U11_NODE_INT_MINOR		169

// S1D13U11_DEV_F defines (device files)
#define	S1D13U11_DEV_F_FB	MKDEV(S1D13U11_NODE_FB_MAJOR,    	S1D13U11_NODE_FB_MINOR)
#define	S1D13U11_DEV_F_TS	MKDEV(S1D13U11_NODE_TS_MAJOR,      S1D13U11_NODE_TS_MINOR)
#define	S1D13U11_DEV_F_INT	MKDEV(S1D13U11_NODE_INT_MAJOR,     S1D13U11_NODE_INT_MINOR)

// S1D13U11_DEV_ID defines (device IDs)
enum {
	S1D13U11_DEV_ID_FB	= 0,	// FRAME BUFFER DEVICE
	S1D13U11_DEV_ID_TS,			// TOUCH SCREEN DEVICE
	S1D13U11_DEV_ID_INT,		// INTERRUPT DEVICE
	S1D13U11_DEV_ID_MAX
};

// S1D13U11_EV_TYPE defines (event types)
enum {
	S1D13U11_EV_TYPE_INT	= 0,		/* interrupt */
	S1D13U11_EV_TYPE_SEQ,				/* complete sequence */
	S1D13U11_EV_TYPE_MAX
};


extern spinlock_t	lcdc_lock;
extern struct semaphore	lcdc_cmdlock;
extern spinlock_t	lcdc_testint_lock;


// callback definition
typedef void (*s1d13u11_callback_t)(int len, unsigned char *data);

// prototypes
extern int s1d13u11_send_cbw(unsigned char ifno, ssize_t len, unsigned char *buf,
						ssize_t rcvlen, unsigned char *rcvbuf);
extern int s1d13u11_recv_csw(unsigned char ifno, ssize_t len, unsigned char *buf);
extern int s1d13u11_send_data(unsigned char ifno, ssize_t len, unsigned char *buf);
extern int s1d13u11_recv_data(unsigned char ifno, ssize_t len, unsigned char *buf);
extern int s1d13u11_send_int(unsigned char ifno, ssize_t len, unsigned char *buf);
extern int s1d13u11_clear_halt(unsigned char ifno );
extern int s1d13u11_soft_reset(unsigned int size, unsigned char *buf );
extern int s1d13u11_suspend( unsigned char ifno );
extern int s1d13u11_resume( unsigned char ifno );
extern int s1d13u11_suspend_status( void );
extern int s1d13u11_busreset( unsigned char ifno );
extern int s1d13u11_control_cmd_send(
			size_t		send_len,
			unsigned char *	send_data,
			size_t		recv_len,
			unsigned char *	recv_data );
int s1d13u11_recv_int(unsigned char id, unsigned int etype, s1d13u11_callback_t *callback);
unsigned char s1d13u11_get_acctype(unsigned char disinc, unsigned char bpp);
void s1d13u11_cnv_size(unsigned char *pcvt, unsigned int raw);

#endif	/* __S1D13U11_H__ */
