//-----------------------------------------------------------------------------
// linux/drivers/usb/misc/epson-13u11/s1d13u11_display.h  
//
// This file is the header for the Epson S1D13U11 display protocols.
// 
// Copyright(C) SEIKO EPSON CORPORATION 2010-2013. All rights reserved.
//
// This driver software is distributed as is, without any warranty of any kind,
// either express or implied as further specified in the GNU Public License. This
// software may be used and distributed according to the terms of the GNU Public
// License, version 2. See the file COPYING in the main directory of this archive
// for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//
//---------------------------------------------------------------------------- 

#ifndef __S1D13U11_DISPLAY_H__
#define __S1D13U11_DISPLAY_H__

// USB_DISP_ID defines
#define	S1D13U11_DISP_ID_VENDOR			0x04B8		/* EPSON */
#define	S1D13U11_DISP_ID_PRODUCT_DL		0x052E		/* USB DISPLAY DOWNLOAD */
#define	S1D13U11_DISP_ID_PRODUCT_MK		0x052F		/* USB DISPLAY MASK ROM */

// S1D13U11_DISP_STAT defines
#define	S1D13U11_DISP_STAT_SUCCESS			0x00
#define	S1D13U11_DISP_STAT_INVALID_PARAM	0x01
#define	S1D13U11_DISP_STAT_COMMAND			0x02
#define	S1D13U11_DISP_STAT_PROTPCOL			0xFF

// S1D13U11_DISP_EVENT defines (event codes)
#define	S1D13U11_DISP_EVENT_INT_STAT_LCDC			0x00
#define	S1D13U11_DISP_EVENT_INT_STAT_SPIINT			0x40
#define	S1D13U11_DISP_EVENT_INT_STAT_SPISEQ			0x41
#define	S1D13U11_DISP_EVENT_INT_STAT_GPI			0x80
#define	S1D13U11_DISP_EVENT_INT_STAT_WUP			0x81
#define	S1D13U11_DISP_EVENT_INT_STAT_KEY			0x90
#define	S1D13U11_DISP_EVENT_INT_STAT_CONNECT		0xF1
#define	S1D13U11_DISP_EVENT_INT_STAT_DISCONNECT		0xF2

// S1D13U11_DISP_IF defines (interface number)
enum {
	S1D13U11_DISP_IF_0			= 0,	/* Interface 0 */
	S1D13U11_DISP_IF_1,					/* Interface 1<Not Use> */
	S1D13U11_DISP_IF_MAX
};
#define	S1D13U11_DISP_IF_IO				S1D13U11_DISP_IF_0
#define	S1D13U11_DISP_IF_IO_STREAM		S1D13U11_DISP_IF_0
#define	S1D13U11_DISP_IF_I2C			S1D13U11_DISP_IF_0
#define	S1D13U11_DISP_IF_SPI			S1D13U11_DISP_IF_0
#define	S1D13U11_DISP_IF_GPIO			S1D13U11_DISP_IF_0
#define	S1D13U11_DISP_IF_INT			S1D13U11_DISP_IF_0
#define S1D13U11_DISP_IF_DISP			S1D13U11_DISP_IF_0
#define S1D13U11_DISP_IF_CMD			S1D13U11_DISP_IF_0

// S1D13U11_DISP_EP defines (endpoints)
enum {
	S1D13U11_DISP_EP_IF_0_BULK_OUT	= 1,
	S1D13U11_DISP_EP_IF_0_BULK_IN,
	S1D13U11_DISP_EP_IF_0_INT,
	S1D13U11_DISP_EP_IF_0_DISP,
	S1D13U11_DISP_EP_IF_0_DISP_OUT,
	S1D13U11_DISP_EP_IF_1_BULK_OUT,		/* Not Used */
	S1D13U11_DISP_EP_IF_1_BULK_IN		/* Not Used */
};

// S1D13U11_DISP_OFFS defines (class protocol offset)
#define	S1D13U11_DISP_OFFS_CBW					0
#define	S1D13U11_DISP_OFFS_CBW_TAG				1
#define	S1D13U11_DISP_OFFS_CBW_DLEN				4
#define	S1D13U11_DISP_OFFS_CBW_DATA				8
#define	S1D13U11_DISP_OFFS_CBW_ACCTYPE			8
#define	S1D13U11_DISP_OFFS_CBW_RDSIZE			12
#define	S1D13U11_DISP_OFFS_CBW_IOSEL			16
#define	S1D13U11_DISP_OFFS_CBW_ADDR				17
#define	S1D13U11_DISP_OFFS_CBW_IO_WRDATA		20
#define	S1D13U11_DISP_OFFS_CBW_I2C_WRDATA		13
#define	S1D13U11_DISP_OFFS_CBW_SPI_WRDATA		14
#define	S1D13U11_DISP_OFFS_CBW_SPI_SEQ_START	10
#define	S1D13U11_DISP_OFFS_CBW_SPI_SEQ_STOP		9
#define	S1D13U11_DISP_OFFS_CSW					0
#define	S1D13U11_DISP_OFFS_CSW_TAG				1
#define	S1D13U11_DISP_OFFS_CSW_DLEN				4
#define	S1D13U11_DISP_OFFS_CSW_DATA				8
#define	S1D13U11_DISP_OFFS_CSW_ACCTYPE			8
#define	S1D13U11_DISP_OFFS_CSW_RDSIZE			12
#define	S1D13U11_DISP_OFFS_CSW_IO_RDDATA		16
#define	S1D13U11_DISP_OFFS_CSW_I2C_RDDATA		8
#define	S1D13U11_DISP_OFFS_CSW_SPI_RDDATA		12
#define	S1D13U11_DISP_OFFS_EVENT_TYPE			0
#define	S1D13U11_DISP_OFFS_EVENT_BLEN			1
#define	S1D13U11_DISP_OFFS_EVENT_INT			2
#define	S1D13U11_DISP_OFFS_GPIO_DATA			3
#define	S1D13U11_DISP_OFFS_SPI_SEQ_DATA			2

// S1D13U11_DISP_H defines (header length)
#define	S1D13U11_DISP_H_CBW_IO_READ			12
#define	S1D13U11_DISP_H_CSW_IO_READ			8
#define	S1D13U11_DISP_H_CBW_IO_WRITE		12
#define	S1D13U11_DISP_H_CSW_IO_WRITE		0
#define	S1D13U11_DISP_H_CBW_I2C_CONFIG		1
#define	S1D13U11_DISP_H_CSW_I2C_CONFIG		0
#define	S1D13U11_DISP_H_CBW_I2C_ACCESS		5
#define	S1D13U11_DISP_H_CSW_I2C_ACCESS		0
#define	S1D13U11_DISP_H_CBW_SPI_START		1
#define	S1D13U11_DISP_H_CSW_SPI_START		0
#define	S1D13U11_DISP_H_CBW_SPI_ACCESS		6
#define	S1D13U11_DISP_H_CSW_SPI_ACCESS		2
#define	S1D13U11_DISP_H_CBW_SPI_SEQ_START	2
#define	S1D13U11_DISP_H_CSW_SPI_SEQ_START	0
#define	S1D13U11_DISP_H_CBW_SPI_SEQ_STOP	1
#define	S1D13U11_DISP_H_CSW_SPI_SEQ_STOP	0
#define	S1D13U11_DISP_H_CBW_GPIO_CONFIG		6
#define	S1D13U11_DISP_H_CSW_GPIO_CONFIG		0
#define	S1D13U11_DISP_H_CBW_GPIO_READ		0
#define	S1D13U11_DISP_H_CSW_GPIO_READ		4
#define	S1D13U11_DISP_H_CBW_GPIO_WRITE		2
#define	S1D13U11_DISP_H_CSW_GPIO_WRITE		0

// S1D13U11_DISP_SEL defines (selectors)
#define	S1D13U11_DISP_SEL_DISINC			0x80		/* DisInc */
#define	S1D13U11_DISP_SEL_ENINC				0x00
#define	S1D13U11_DISP_SEL_ACCTYPE_8			0x01		/* AccType */
#define	S1D13U11_DISP_SEL_ACCTYPE_16		0x02
#define	S1D13U11_DISP_SEL_ACCTYPE_32		0x04
#define S1D13U11_DISP_SEL_ACCTYPE_MASK		0x7F
#define	S1D13U11_DISP_SEL_LCD_1				0xF0		/* IOSel */
#define	S1D13U11_DISP_SEL_LCD_2				0xD0
#define	S1D13U11_DISP_SEL_I2C_RATE_50K		0x01		/* TransferRate */
#define	S1D13U11_DISP_SEL_I2C_RATE_100K		0x02
#define	S1D13U11_DISP_SEL_I2C_RATE_200K		0x03
#define	S1D13U11_DISP_SEL_I2C_RATE_400K		0x04
//#define	S1D13U11_DISP_SEL_SPI_IN		0x00		/* PHASE */
//#define	S1D13U11_DISP_SEL_SPI_OUT		0x80

#define	S1D13U11_DISP_SEL_SPI_CPHA_ACT		0x00		// (SPI_CONFIG)CPHA
#define	S1D13U11_DISP_SEL_SPI_CPHA_UNACT	0x80		// (SPI_CONFIG)CPHA
#define	S1D13U11_DISP_SEL_SPI_CPOL_HIGH		0x00		// (SPI_CONFIG)CPOL
#define	S1D13U11_DISP_SEL_SPI_CPOL_LOW		0x40		// (SPI_CONFIG)CPOL
#define	S1D13U11_DISP_SEL_SPI_SS_NON0		0x00		// (SPI_CONFIG)SS
#define	S1D13U11_DISP_SEL_SPI_SS_NON1		0x10		// (SPI_CONFIG)SS
#define	S1D13U11_DISP_SEL_SPI_SS_HIGH		0x20		// (SPI_CONFIG)SS
#define	S1D13U11_DISP_SEL_SPI_SS_LOW		0x30		// (SPI_CONFIG)SS
#define	S1D13U11_DISP_SEL_SPI_INT_NON0		0x00		// (SPI_CONFIG)INT
#define	S1D13U11_DISP_SEL_SPI_INT_NON1		0x04		// (SPI_CONFIG)INT
#define	S1D13U11_DISP_SEL_SPI_INT_LOW		0x08		// (SPI_CONFIG)INT
#define	S1D13U11_DISP_SEL_SPI_INT_HIGH		0x0C		// (SPI_CONFIG)INT
#define	S1D13U11_DISP_SEL_SPI_LSB_MSB		0x00		// (SPI_CONFIG)LSB
#define	S1D13U11_DISP_SEL_SPI_LSB_LSB		0x02		// (SPI_CONFIG)LSB
#define	S1D13U11_DISP_SEL_SPI_RATE_30M		0x00		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_15M		0x01		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_7p5M		0x02		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_3p75M	0x03		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_1p87M	0x04		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_937K		0x05		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_468K		0x06		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_234K		0x07		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_117K		0x08		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_58p5K	0x09		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_29p2K	0x0A		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_14p6K	0x0B		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_7p3K		0x0C		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_3p6K		0x0D		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_RATE_1p8K		0x0E		// (SPI_CONFIG)RATE
#define	S1D13U11_DISP_SEL_SPI_SSM_SINGLE	0x00		// (SPI_CONFIG)SS_Mode
#define	S1D13U11_DISP_SEL_SPI_SSM_MULTI		0x01		// (SPI_CONFIG)SS_Mode

#define	S1D13U11_DISP_SEL_SPI_SEQ_CH0		0x00		// (SPI_SEQUENCE_START/STOP)bChannel
#define	S1D13U11_DISP_SEL_SPI_SEQ_CH1		0x01		// (SPI_SEQUENCE_START/STOP)bChannel
#define S1D13U11_DISP_SEL_SPI_SEQ_WAIT_DLY	0x00		// (SPI_SEQUENCE_START)bMode
#define S1D13U11_DISP_SEL_SPI_SEQ_WAIT_IRQ	0x01		// (SPI_SEQUENCE_START)bMode
#define	S1D13U11_DISP_SEL_SPI_SEQ_MIN		0x01		// (SPI_SEQUENCE_START)wSequenceSize
#define	S1D13U11_DISP_SEL_SPI_SEQ_MAX		0xFF		// (SPI_SEQUENCE_START)wSequenceSize
#define	S1D13U11_DISP_SEL_SPI_SEQ_WRITE		0x00		// (SPI_SEQUENCE_START)Sequence
#define	S1D13U11_DISP_SEL_SPI_SEQ_READ		0x01		// (SPI_SEQUENCE_START)Sequence
#define	S1D13U11_DISP_SEL_SPI_SEQ_SS_ASS	0x02		// (SPI_SEQUENCE_START)Sequence
#define	S1D13U11_DISP_SEL_SPI_SEQ_SS_NEG	0x03		// (SPI_SEQUENCE_START)Sequence
#define	S1D13U11_DISP_SEL_SPI_SEQ_INT_DIS	0x04		// (SPI_SEQUENCE_START)Sequence
#define	S1D13U11_DISP_SEL_SPI_SEQ_INT_ENB	0x05		// (SPI_SEQUENCE_START)Sequence
#define	S1D13U11_DISP_SEL_SPI_SEQ_WAIT		0x06		// (SPI_SEQUENCE_START)Sequence

#define	S1D13U11_DISP_SEL_GPIO_BMGPIOA_DIR	0x00		// (GPIO_CONFIG)bmGPIOA_Dir
#define	S1D13U11_DISP_SEL_GPIO_BMGPIOB_DIR	0x00		// (GPIO_CONFIG)bmGPIOB_Dir
#define	S1D13U11_DISP_SEL_GPIO_BMGPIOA_PULL	0xFF		// (GPIO_CONFIG)bmGPIOA_PullUp
#define	S1D13U11_DISP_SEL_GPIO_BMGPIOB_PULL	0xFF		// (GPIO_CONFIG)bmGPIOB_PullUp
#define	S1D13U11_DISP_SEL_GPIO_SCANLINE		0x08		// (GPIO_CONFIG)bKeyScanLine

#define	S1D13U11_DISP_SEL_KEYSCAN_ENABLE_ON	0x01		// (KEYSCAN_CONTROL)bKeyScanEnable
#define	S1D13U11_DISP_SEL_KEYSCAN_ENABLE_OFF 0x00		// (KEYSCAN_CONTROL)bKeyScanEnable
#define	S1D13U11_DISP_SEL_KEYSCAN_CLOCK		0x00		// (KEYSCAN_CONTROL)bKeyScanClock
#define	S1D13U11_DISP_SEL_KEYSCAN_SCLOCK	0x03		// (KEYSCAN_CONTROL)bKeyScanSamplingClock
#define	S1D13U11_DISP_SEL_KEYSCAN_INTERVAL	0x03		// (KEYSCAN_CONTROL)bKeyScanInterval

//#define S1D13U11_DISP_SEL_SPI_SEQ_WAIT_DLY	0x00
//#define S1D13U11_DISP_SEL_SPI_SEQ_WAIT_IRQ	0x10
//#define S1D13U11_DISP_SEL_SPI_SEQ_WAIT_NOP	0x20
//#define S1D13U11_DISP_SEL_SPI_CS_SINGLE		0x00		/* CS mode */
//#define S1D13U11_DISP_SEL_SPI_CS_MULTI		0x01
//#define S1D13U11_DISP_SEL_GPIO_TRIG_EDGE		0x00		/* TriggerMode */
//#define S1D13U11_DISP_SEL_GPIO_TRIG_LEVEL		0x01
//#define S1D13U11_DISP_SEL_GPIO_DISEN_EDGE		0x00		/* EdgeEnable */
//#define S1D13U11_DISP_SEL_GPIO_EN_EDGE		0x01

// S1D13U11_DISP_ADDR defines
#define	S1D13U11_DISP_ADDR_I2C_SLAVE_DEF	0
#define	S1D13U11_DISP_ADDR_I2C_SLAVE		S1D13U11_DISP_ADDR_I2C_SLAVE_DEF

// S1D13U11_DISP_LIMIT defines
#define	S1D13U11_DISP_LIMIT_FB_ADDR_L	0x000000
#define	S1D13U11_DISP_LIMIT_FB_ADDR_H	0x0FFFFF

// S1D13U11_DISP_GPIO defines (GPIO information)
enum {
	S1D13U11_DISP_GPIO_INFO_IH	= 0,
	S1D13U11_DISP_GPIO_INFO_IL,
	S1D13U11_DISP_GPIO_INFO_OH,
	S1D13U11_DISP_GPIO_INFO_OL,
	S1D13U11_DISP_GPIO_INFO_LEN
};


//=======================================================================
//	Fundamental model
//=======================================================================

//  s1d13u11_display_cbw (CBW)
struct s1d13u11_disp_cbw {
	unsigned char	cmd;			/* CBWCB */
	unsigned char	tag;			/* CBWTag */
	unsigned char	rsv[2];			/* CBWReserved */
};

// s1d13u11_display_csw (CSW)
struct s1d13u11_disp_csw {
	unsigned char	stat;			/* CSWStatus */
	unsigned char	tag;			/* CSWTag */
	unsigned char	reserv[2];		/* reserved */
	unsigned short	len;			/* CSWDataLength */
	unsigned char	parm[2];		/* Parameter */
};

// s1d13u11_disp_event (event block)
struct s1d13u11_disp_event {
	unsigned char	etype;			/* EventType */
	unsigned char	eReserv ;		/* Reserved */
	unsigned short	elen;			/* Length */
};


//===<CFG_GETINFO>=======================================================

// CFG_GETINFO data send
struct s1d13u11_disp_cbw_cfg_getinfo {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		para[12];	// Parameter
};

// CFG_GETINFO data receive
struct s1d13u11_disp_csw_cfg_getinfo {
	struct s1d13u11_disp_csw	csw;
	unsigned char		bMode;		// S1D13U11_DISP_BMODE_
	unsigned char		dummy00;
	unsigned short		wVersion ;
};

#define	S1D13U11_DISP_BMODE_NONAPPLI	0x00
#define	S1D13U11_DISP_BMODE_APPLI		0x01


//===<CFG_DOWNLOAD>=============================================================

// CFG_DOWNLOAD data send
struct s1d13u11_disp_cbw_cfg_download {
	struct s1d13u11_disp_cbw	cbw;
	unsigned long		dwOffset;	// S1D13U11_DISP_DWOFFSET
	unsigned long		dwSize;
	unsigned char		dummy00[4];
};
#define	S1D13U11_DISP_DWOFFSET		0x00

// CFG_DOWNLOAD data receive
struct s1d13u11_disp_csw_cfg_download {
	struct s1d13u11_disp_csw	csw;
};


//===<CFG_SWITCH>===============================================================

// CFG_SWITCH data send
struct s1d13u11_disp_cbw_cfg_switch {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		dummy00[12];
};

// CFG_SWITCH data receive
struct s1d13u11_disp_csw_cfg_switch {
	struct s1d13u11_disp_csw	csw;
};


//===<SPI_CONFIG>===============================================================

// SPI_CONFIG data send
struct s1d13u11_disp_cbw_spi_config {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		ch0;
	unsigned char		ch0Rate;
	unsigned char		ch0Mode;
	unsigned char		dummy00;
	unsigned char		ch1;
	unsigned char		ch1Rate;
	unsigned char		ch1Mode;
	unsigned char		dummy01[5];
};

// SPI_CONFIG data receive
struct s1d13u11_disp_csw_spi_config {
	struct s1d13u11_disp_csw	csw;
};


//===<SPI_ACCESS>===============================================================

// SPI_ACCESS data send
struct s1d13u11_disp_cbw_spi_access {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		bChannel;
	unsigned char		dummy00[3];
	unsigned short		wWriteSize;
	unsigned char		dummy01[2];
	unsigned short		wReadSize;
	unsigned char		dummy03[2];
};

// SPI_ACCESS data receive
struct s1d13u11_disp_csw_spi_access {
	struct s1d13u11_disp_csw	csw;
};


//===<SPI_SEQUENCE_START>=======================================================

// SPI_SEQUENCE_START data send
struct s1d13u11_disp_cbw_spi_sequence_start {
	struct s1d13u11_disp_cbw	cbw ;
	unsigned char		bChannel ;
	unsigned char		bMode ;
	unsigned char		bCycleTime;
	unsigned char		dummy00[1];
	unsigned short		size;
	unsigned char		dummy01[6];
//	unsigned char		*seqData ;
};

// SPI_SEQUENCE_START data receive
struct s1d13u11_disp_csw_spi_sequence_start {
	struct s1d13u11_disp_csw	csw;
};


//===<SPI_SEQUENCE_STOP>========================================================

// SPI_SEQUENCE_STOP data send
struct s1d13u11_disp_cbw_spi_sequence_stop {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		bChannel;
	unsigned char		dummy00[11];
};

// SPI_SEQUENCE_STOP data receive
struct s1d13u11_disp_csw_spi_sequence_stop {
	struct s1d13u11_disp_csw	csw;
};


//===<LCDC_VRAM_ACC_ENABLE>=====================================================

// LCDC_VRAM_ACC_ENABLE data send
struct s1d13u11_disp_cbw_lcdc_vram_acc_enable {
	struct s1d13u11_disp_cbw	cbw;
	unsigned long		pictureSize;
	unsigned char		dummy00[8];
};

// LCDC_VRAM_ACC_ENABLE data receive
struct s1d13u11_disp_csw_lcdc_vram_acc_enable {
	struct s1d13u11_disp_csw	csw;
};


//===<LCDC_VRAM_ACC_DISABLE>====================================================

// LCDC_VRAM_ACC_DISABLE data send
struct s1d13u11_disp_cbw_lcdc_vram_acc_disable {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		dummy00[12];
};

// LCDC_VRAM_ACC_DISABLE data receive
struct s1d13u11_disp_csw_lcdc_vram_acc_disable {
	struct s1d13u11_disp_csw	csw;
};


//===<LCDC_WRITE>===============================================================

// LCDC_WRITE data send
struct s1d13u11_disp_cbw_lcdc_write {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		dummy00[4];
	unsigned short		size ;
	unsigned char		dummy01[6];
};

// LCDC_WRITE data receive
struct s1d13u11_disp_csw_lcdc_write {
	struct s1d13u11_disp_csw	csw;
};


//===<LCDC_READ>================================================================

// LCDC_READ data send
struct s1d13u11_disp_cbw_lcdc_read {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		dummy00[2];
	unsigned char		disInc ;
	unsigned char		dummy01[5];
	unsigned short		rSize ;
	unsigned short		rAdd ;
};

// LCDC_READ data receive
struct s1d13u11_disp_csw_lcdc_read {
	struct s1d13u11_disp_csw	csw;
};


//===<GPIO_CONFIG>==============================================================

// GPIO_CONFIG data send
struct s1d13u11_disp_cbw_gpio_config {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		bmGPIOA_Dir;
	unsigned char		bmGPIOB_Dir;
	unsigned char		bmGPIOA_PullUp;
	unsigned char		bmGPIOB_PullUp;
	unsigned char		bKeyScanLine;
	unsigned char		dummy00[7];
};

// GPIO_CONFIG data receive
struct s1d13u11_disp_csw_gpio_config {
	struct s1d13u11_disp_csw	csw;
};


//===<WAKEUP_ON_CONFIG>=========================================================

// WAKEUP_ON_CONFIG data send
struct s1d13u11_disp_cbw_wakeup_config {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		dummy00;
	unsigned char		bType;
	unsigned char		dummy01[2];
	unsigned short		wWriteSize;
	unsigned char		dummy02[6];
};

// GPIO_CONFIG data receive
struct s1d13u11_disp_csw_wakeup_config {
	struct s1d13u11_disp_csw	csw;
};


//===<EVENT_INT_CONTROL>==========================================================

// EVENT_INT_CONTROL data send
struct s1d13u11_disp_cbw_event_int_control {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		bType;
	unsigned char		dummy00[11];
};

// EVENT_INT_CONTROL data receive
struct s1d13u11_disp_csw_event_int_control {
	struct s1d13u11_disp_csw	csw;
};


//===<KEYSCAN_CONTROL>==========================================================

// KEYSCAN_CONTROL data send
struct s1d13u11_disp_cbw_keyscan_control {
	struct s1d13u11_disp_cbw	cbw;
	unsigned char		bKeyScanEnable;
	unsigned char		dummy00[1];
	unsigned char		bKeyScanClock;
	unsigned char		bKeyScanSamplingClock;
	unsigned char		bKeyScanInterval;
	unsigned char		dummy01[7];
};

// KEYSCAN_CONTROL data receive
struct s1d13u11_disp_csw_keyscan_control {
	struct s1d13u11_disp_csw	csw;
};

// KEYSCAN_EVENT data receive
struct s1d13u11_disp_event_keyscan_event {
	struct s1d13u11_disp_event	evt;
	unsigned char		scandat[8];
};


//===<SPI_SEQUENCE_EVENT>=======================================================

// SPI_SEQUENCE_EVENT data send
struct s1d13u11_disp_event_spiseq_event {
	struct s1d13u11_disp_event	evt;
	unsigned char		bChannel;
	unsigned char		dummy00[1];
};


#endif  /* __S1D13U11_DISPLAY_H__ */
