//-----------------------------------------------------------------------------
// linux/drivers/usb/misc/epson-13u11/s1d13u11_fb.h  
//
// This file is the header for the Epson S1D13U11 frame buffer driver.
// 
// Copyright(C) SEIKO EPSON CORPORATION 2010-2013. All rights reserved.
//
// This driver software is distributed as is, without any warranty of any kind,
// either express or implied as further specified in the GNU Public License. This
// software may be used and distributed according to the terms of the GNU Public
// License, version 2. See the file COPYING in the main directory of this archive
// for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//
//---------------------------------------------------------------------------- 

#ifndef __S1D13U11_FB_H__
#define __S1D13U11_FB_H__

#include <linux/kernel.h>
#include <linux/fb.h>
#include <linux/types.h>

#include "s1d13u11.h"

//#define	S1D13U11_FB_DEF_VAR_BPP		16		// bits per pixel (16/24/32)
#define	S1D13U11_FB_DEF_VAR_BPP		24		// bits per pixel (16/24/32)
#define	S1D13U11_FB_SIZE_X			320		// panel width
#define	S1D13U11_FB_SIZE_Y			240		// panel height
#define S1D13U11_FB_PALETTE_SIZE	256
#define S1D13U11_FB_VFB_SIZE_BYTES	S1D13U11_FB_SIZE_X*S1D13U11_FB_SIZE_Y*S1D13U11_FB_DEF_VAR_BPP/8

#if S1D13U11_FB_DEF_VAR_BPP == 16
#define	S1D13U11_FB_RED_BIT		{ 11, 5, 0 }
#define	S1D13U11_FB_GREEN_BIT	{  5, 6, 0 }
#define	S1D13U11_FB_BLUE_BIT	{  0, 5, 0 }
#elif S1D13U11_FB_DEF_VAR_BPP == 24
#define	S1D13U11_FB_RED_BIT		{ 16, 8, 0 }
#define	S1D13U11_FB_GREEN_BIT	{  8, 8, 0 }
#define	S1D13U11_FB_BLUE_BIT	{  0, 8, 0 }
#elif S1D13U11_FB_DEF_VAR_BPP == 32
#define	S1D13U11_FB_RED_BIT		{  0, 8, 0 }
#define	S1D13U11_FB_GREEN_BIT	{ 24, 8, 0 }
#define	S1D13U11_FB_BLUE_BIT	{ 16, 8, 0 }
#endif

// S1D13U11_FB_DEF defines (frame buffer defaults)
#define	S1D13U11_FB_DEF_SIZE_BYTEPP		S1D13U11_FB_DEF_VAR_BPP / 8	/* byte per pixel */
//#define S1D13U11_FB_DEF_REFRESH_TIME_MIN	500
//#define S1D13U11_FB_DEF_REFRESH_TIME_MAX	10000
//#define S1D13U11_FB_DEF_TIMER_EXPIRES	20
//#define S1D13U11_FB_DEF_TIMER_START		20
//#define S1D13U11_FB_DEF_TIMER_PUSH		20


// S1D13U11_FB_SIZE defines  
//#define	S1D13U11_FB_SIZE_HEAD		32	/* enough for CBW or CSW */
//#define S1D13U11_FB_SIZE_ORDER_WR	9	/* S1D13U11_FB_SIZE_BUF_WR < 4KB * 2^10 */
#define	S1D13U11_FB_SIZE_WR_DATA		(S1D13U11_FB_SIZE_X * S1D13U11_FB_SIZE_Y * S1D13U11_FB_DEF_SIZE_BYTEPP)


struct s1d13u11fb_par 
{
	volatile unsigned char *RegAddr; 
	unsigned RegAddrMappedSize;
	int blank_mode;
	u32 pseudo_palette[16];
	int dirtyrange;
	u32 dx1,dx2,dy1,dy2;
};

extern struct fb_fix_screeninfo	s1d13u11fb_fix;

extern spinlock_t	lcdc_lock;
extern spinlock_t	lcdc_testint_lock;

// prototypes
int s1d13u11fb_init(void);
void __exit s1d13u11fb_exit(void);
 
int s1d13u11fb_check_var(struct fb_var_screeninfo *var, struct fb_info *info);
int s1d13u11fb_setcolreg(unsigned regno, unsigned red, unsigned green, unsigned blue, unsigned transp, struct fb_info *info);
int s1d13u11fb_ioctl(struct fb_info *info, unsigned int cmd, unsigned long arg );
void s1d13u11fb_CleanUp(void);

// following parameters are for initializing 320x240 16bpp panels with S5U13U11P00C100 board.
static unsigned short	s_initData[] = {
	0x006E,		0x0003,	// General Purpose Output Register 1
	0x0004,		0x0001,	// PLL D-Divider Register
	0x0006,		0x002A,	// PLL Setting Register 0
	0x000C,		0x000A,	// PLL N-Divider Register
	0x000E,		0x0080,	// SS Control Register 0
	0x0010,		0x0055,	// SS Control Register 1
	0x0012,		0x00C2,	// Clock Source Select Register
	//0x0014,		0x0005,	// LCD Panel Type Register(RGB565 ALCDIF=18Bit)
	0x0014,		0x0041,	// LCD Panel Type Register(RGB565 ALCDIF=18Bit)
				// RGB[565], RGB[888(24Bit)], RGB[888(32Bit)]
	0x0016,		0x0027,	// 320 :Horizontal Display Width Register (HDISP)
	0x0018,		0x004F,	// 160 :Horizontal Non-Display Period Register (HNDP)
	0x001A,		0x00EF,	// 240 :Vertical Display Height Register 0 (VDISP)
	0x001C,		0x0000,	// 240 :Vertical Display Height Register 1 (VDISP)
	0x001E,		0x0016,	// 46  :Vertical Non-Display Period Register (VNDP)
	0x0020,		0x000A,	// PHS Pulse Width Register (HSW)
	0x0022,		0x0076,	// PHS Pulse Start Position Register (HPS)
	0x0024,		0x0002,	// PVS Pulse Width Register (VSW)
	0x0026,		0x0028,	// PVS Pulse Start Position Register (VPS)
	0x0028,		0x0000,	// PCLK Polarity Register
	0x0082,		0x0003,	// SDRAM Control Register 0
	0x008C,		0x00FF,	// SDRAM Refresh Counter Register 0
	0x008E,		0x0003,	// SDRAM Refresh Counter Register 1
	0x0090,		0x0050,	// SDRAM Write Buffer Memory Size Register 0
	0x0004,		0x0081,	// PLL D-Divider Registerr
	0x006E,		0x0001,	// General Purpose Output Register 1
	0x0012,		0x00C2,	// Clock Source Select Register
	0x0068,		0x00E8,	// Power Save Register
	0x0068,		0x0000,	// Power Save Register
	0x0068,		0x0001,	// Power Save Register
	0x0084,		0x0000,	// SDRAM Status Register 0
	0x0084,		0x0082,	// SDRAM Status Register 0
	0x002A,		0x0000,	// Display Mode Register
	0x0052,		0x0000,	// Input Mode Register

	0x005A,		0x0010,	// Write Window X Start Position Register Window(0,0,319,239)
	0x005C,		0x0016,	// Write Window Y Start Position Register 0
	0x005E,		0x0002,	// Write Window Y Start Position Register 1
	0x0060,		0x003C,	// Write Window X End Position Register
	0x0062,		0x0026,	// Write Window Y End Position Register 0
	0x0064,		0x0002,	// Write Window Y End Position Register 1

	0x002A,		0x0001,	// REG[2Ah] Display Mode Register
	0x0050,		0x0080,	// REG[50h] Display Control Register
	0x006E,		0x0000,	// General Purpose Output Register 1
	0x00B6,		0x0001,	// Interrupt Clear Register
	0x00B6,		0x0000,	// Interrupt Clear Register
	0x00B2,		0x0001,	// Interrupt Control Register
	0x0098,		0x004F,	// Alpha-blending Horizontal Size Register
	0x009A,		0x003B,	// Alpha-blending Vertical Size Register 0
	0x009C,		0x0003,	// Alpha-blending Vertical Size Register 1
	0x009E,		0x0060,	// Alpha-blending Value Register
	0x00A0,		0x0000,	// Alpha-blending Input 1 Start Address Register 0
	0x00A2,		0x0000,	// Alpha-blending Input 1 Start Address Register 1
	0x00A4,		0x0000,	// Alpha-blending Input 1 Start Address Register 2
	0x00AC,		0x0000,	// Alpha-blending Output Start Address Register 0
	0x00AE,		0x0000,	// Alpha-blending Output Start Address Register 1
	0x00B0,		0x0000,	// Alpha-blending Output Start Address Register 2

	0x0054,		0x00FF,	// Transparency Key Color Red Register
	0x0056,		0x00FF,	// Transparency Key Color Green Register
	0x0058,		0x00FF,	// Transparency Key Color Blue Register

	0x0094,		0x0001,	// Alpha-blending Control Register
	0x0094,		0x0000,	// Alpha-blending Control Register

	0x00B4,		0x0001,	// Interrupt Status Register
	0x00B6,		0x0001,	// Interrupt Clear Register
	0x00B6,		0x0000,	// Interrupt Clear Register

	0x0072,		0x004B,	// PWM High Duty Register 0 (150 count)
	0x0074,		0x0000,	// PWM High Duty Register 1
	0x0076,		0x0000,	// PWM High Duty Register 2
	0x0078,		0x0000,	// PWM High Duty Register 3
	0x007A,		0x0096,	// PWM Low Duty Register 0 (150 count)
	0x007C,		0x0000,	// PWM Low Duty Register 1
	0x007E,		0x0000,	// PWM Low Duty Register 2
	0x0080,		0x0000,	// PWM Low Duty Register 3
	0x0070,		0x0086	// PWM Control Register(PWM On)
} ;


static unsigned short	s_initLCDData[] = {
	0x0052,		0x0000,	// Input Mode Register
	0x005A,		0x0000, // Write Window X Start Position Register
	0x005C,		0x0000, // Write Window Y Start Position Register 0
	0x005E,		0x0000, // Write Window Y Start Position Register 1
	0x0060,		0x004F,	// Write Window X End Position Register
	0x0062,		0x003B,	// Write Window Y End Position Register 0
	0x0064,		0x0003,	// Write Window Y End Position Register 1
	0x002A,		0x0001,	// Display Mode Register
	0x0050,		0x0080	// Display Control Register
} ;


// initializes a 320x240 PIP window
// add in if initialization is required
/* 
static unsigned short	s_initLCDDataPIP[] = {
	0x0052,		0x0000, // Input Mode Register
	0x005A,		0x0000, // Write Window X Start Position Register
	0x005C,		0x0000, // Write Window Y Start Position Register 0
	0x005E,		0x0000, // Write Window Y Start Position Register 1
	0x0060,		0x004F,	// Write Window X End Position Register
	0x0062,		0x003B,	// Write Window Y End Position Register 0
	0x0064,		0x0003,	// Write Window Y End Position Register 1
	0x002A,		0x0005,	// Display Mode Register
	0x0050,		0x0080	// Display Control Register
} ;
*/


static unsigned short	s_setData[] = {
	//0x0014,		0x0007,	// LCD Panel Type Register(RGB565 ALCDIF=18Bit)
	0x0014,		0x0041,	// LCD Panel Type Register(RGB565 ALCDIF=18Bit)
				// RGB[565], RGB[888(24Bit)], RGB[888(32Bit)]
} ;


#endif  /* __S1D13U11_FB_H__ */
