//-----------------------------------------------------------------------------
// linux/drivers/usb/misc/epson-13u11/test/13u11test.c  
//
// This file demonstrates some of the function available when using the 
// Epson S1D13U11 virtual frame buffer driver.
// 
// Copyright(C) SEIKO EPSON CORPORATION 2010-2013. All rights reserved.
//
// This driver software is distributed as is, without any warranty of any kind,
// either express or implied as further specified in the GNU Public License. This
// software may be used and distributed according to the terms of the GNU Public
// License, version 2. See the file COPYING in the main directory of this archive
// for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//
//  Compile with:
//        gcc -s -Wall -Wstrict-prototypes 13u11test.c -o 13u11test
//
//-----------------------------------------------------------------------------
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <linux/fb.h>
#include <fcntl.h>
#include <math.h>
#include <errno.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <linux/input.h>

#include "../s1d13u11_ctl.h"


//#define		DEF_FB_DEVICE_NAME	"/dev/fb0"
#define		DEF_FB_DEVICE_NAME	"/dev/fb1"
//#define		DEF_TS_DEVICE_NAME	"/dev/input/event2"
//#define		DEF_TS_DEVICE_NAME	"/dev/input/event3"
//#define		DEF_TS_DEVICE_NAME	"/dev/input/event4"
#define		DEF_TS_DEVICE_NAME	"/dev/input/event5"

#define SIZE_X	320
#define SIZE_Y	240
#define BITPP	16			// 16 = 16bpp, 24 = 24bpp mode1, 32 = 24bpp mode2
#define BYTEPP	(BITPP / 8)

#define SAMPLE_01	"sample01.bmp"
#define SAMPLE_02	"sample02.bmp"

#define __DEBUG
#ifdef __DEBUG
#define DPRINT(...)	printf(__VA_ARGS__)
#else
#define DPRINT(...)
#endif


//----------------------------------------------------------------------
// Structures
//----------------------------------------------------------------------
typedef struct tag_BITMAPFILEHEADER {
#define BITMAPFILEHEADER_SIZE	14
  char header[BITMAPFILEHEADER_SIZE];
} BITMAPFILEHEADER ;

typedef struct tagBITMAPINFOHEADER{
    unsigned long  biSize;
    long           biWidth;
    long           biHeight;
    unsigned short biPlanes;
    unsigned short biBitCount;
    unsigned long  biCompression;
    unsigned long  biSizeImage;
    long           biXPixPerMeter;
    long           biYPixPerMeter;
    unsigned long  biClrUsed;
    unsigned long  biClrImporant;
} BITMAPINFOHEADER;


//----------------------------------------------------------------------
// Grobal Variables
//----------------------------------------------------------------------
static int				s_fb_fd;
static int				s_ts_fd;
static char*			_fb_addr = 0;
static BITMAPFILEHEADER	bitFileHead;
static BITMAPINFOHEADER	bitInfoHead;

//----------------------------------------------------------------------
// Prototypes
//----------------------------------------------------------------------
static int display_bitmap( char* fileName , int x, int y,  int width, int height);
static int changeBitmapToVRAMData( unsigned int width, unsigned int height, unsigned int size, unsigned char *buf );

/* ========================================================================= */
/**
 * @brief	- S1D13U11 sample application: showbmp
 *
 * @par		usage:
 * @par		Note: 
 *
 * @param	argc;	not used
 * @param	argv;	not used
 **/
/* ========================================================================= */
int main( int argc, char *argv[] )
{
	struct s1d13u11_ctl pctl;
	int ii = 0 ;
	unsigned short WriteData[20] ;
	struct fb_var_screeninfo vinfo;
	struct fb_fix_screeninfo finfo;
	long int screensize = 0;
	
	//touchscreen variables
	struct input_event ts_event[8], *event, *last_event;	//touchscreen event
	int x=0, y=0, rt=0, button=0;		// x,y position, rt=pressure, button state
	int tslen=0, tsvalue=0;

	//print some initial information about the display
	printf("%dx%d %dbpp mode=%d\n", SIZE_X, SIZE_Y, (BITPP != 32) ? BITPP : 24,
		 (BITPP == 16) ? 0 : ((BITPP % 16) ? 1 : 2) );
	
	// open frame buffer
	s_fb_fd = open(DEF_FB_DEVICE_NAME, O_RDWR);
	if( s_fb_fd < 0 ){
		printf( "Error Frame Buffer Device![%d]\n",s_fb_fd );
		return -1 ;
	}

	// open touchscreen
	s_ts_fd = open(DEF_TS_DEVICE_NAME,O_RDONLY);
	if (s_ts_fd < 0){
		close( s_fb_fd );		//close frame buffer
		printf( "Error Touchscreen Device![%d]\n",s_ts_fd );
		return -1;
	}

	// Get fixed screen information
	if (ioctl(s_fb_fd, FBIOGET_FSCREENINFO, &finfo))
	{
		printf("Error reading fixed information.\n");
		exit(2);
	}

	// Get variable screen information
	if (ioctl(s_fb_fd, FBIOGET_VSCREENINFO, &vinfo))
	{
		printf("Error reading variable information.\n");
		exit(3);
	}
	
	printf("\n\nThe framebuffer device was opened successfully.\n");
	printf("%dx%d, %dbpp, xoffset:%x, yoffset:%x\n", vinfo.xres, vinfo.yres, vinfo.bits_per_pixel,vinfo.xoffset,vinfo.yoffset);

	// Figure out the size of the screen in bytes
	screensize = vinfo.xres * vinfo.yres * vinfo.bits_per_pixel / 8;
	
	// Basic information output
	printf("Frame buffer mem start: %lXh\n", finfo.smem_start);
	printf("Frame buffer mem length (in bytes): %u \n", finfo.smem_len);
	printf("Line length (in bytes): %u\n", finfo.line_length);
	printf("Screensize (in bytes): %u\n", (unsigned) screensize);
	printf("Grayscale: %d\n", vinfo.grayscale);

	// frame buffer map
	_fb_addr = (char *)mmap(0,
				SIZE_X * SIZE_Y * BYTEPP,
				PROT_READ | PROT_WRITE, MAP_SHARED,
				s_fb_fd,
				0);

	printf("Testing register writes...\n");
	printf("Press any key to turn the display off.\n" );
	getchar();
	
	// Clear WriteData
	memset( (void *)&WriteData, 0x00, sizeof(WriteData) );

	WriteData[ii++] = 0x2A ;	// REG[2Ah]
	WriteData[ii++] = 0x00 ;	// Display Off
	pctl.size = ii * sizeof(short);
	pctl.buf = (unsigned char *) &WriteData;
	if (ioctl(s_fb_fd, S1D13U11_FB_CMD_LCDC_WRITE, &pctl)){
		printf("Write Failed.\n");
		exit(4);
	}
	
	printf( "Display off, press any key to turn on.\n" );
	getchar();
	
	// Clear WriteData
	memset( (void *)&WriteData, 0x00, sizeof(WriteData) );

	WriteData[ii++] = 0x2A ;	// REG[2Ah]
	WriteData[ii++] = 0x01 ;	// Display On
	pctl.size = ii * sizeof(short);
	pctl.buf = (unsigned char *) &WriteData;
	if (ioctl(s_fb_fd, S1D13U11_FB_CMD_LCDC_WRITE, &pctl)){
		printf("Write Failed.\n");
		exit(4);
	}

	printf("Display on, press any key to test register reads...\n");
	getchar();
	
	pctl.addr = 0x00;
	pctl.size = 0x02;
	
	if (ioctl(s_fb_fd, S1D13U11_FB_CMD_LCDC_READ, &pctl)){
		printf("Read fails.\n");
		exit(4);
	}
	
	if (pctl.value == 0xBC){
		printf("Register Read Passes:\n");
		printf("  Product Code: REG[00h] = %Xh\n\n", pctl.value);
	}
	else{
		printf("Register Read Fails\n");
		exit(4);
	}
	
	printf( "Press any key to display test image 1\n" );
	getchar();

	// display test image 1
	display_bitmap( SAMPLE_01, 0, 0, SIZE_X, SIZE_Y );
	// flush VFB
	if (ioctl(s_fb_fd, S1D13U11_FB_CMD_FLUSH_VFB, NULL)){
		printf("Error flushing VFB.\n");
		exit(4);
	}
	
	printf( "Press any key to display test image 2\n" );
	getchar();
	
	// display test image 1
	display_bitmap( SAMPLE_02, 0, 0, SIZE_X, SIZE_Y );
	// flush VFB
	if (ioctl(s_fb_fd, S1D13U11_FB_CMD_FLUSH_VFB, NULL)){
		printf("Error flushing VFB.\n");
		exit(4);
	}
	
	//demonstrate touchscreen (touch lower right corner to exit)
	printf("Touch screen to report current x,y and pressure status.\n");
	printf("Touch the lower right corner to stop reporting.\n\n");
	while(x<3500 || y<3500){
		tslen = read(s_ts_fd, &ts_event, sizeof( ts_event));
		event = ts_event;
		last_event = (void *) ts_event + tslen;

		for (event = ts_event; event < last_event; event++){
			tsvalue = event->value;

			switch (event->type) {
			case EV_ABS:
				switch (event->code) {
				case ABS_X:
					x = tsvalue;
					break;
				case ABS_Y:
					y = tsvalue;
					break;
				case ABS_PRESSURE:
					rt = tsvalue;
					break;
				}

			case EV_KEY:
				if (tsvalue == 2)
					break;
				switch (event->code) {
				case BTN_TOUCH:
					button = tsvalue;
					break;
				}
			}
		}
		printf("Button=%d, X=%d, Y=%d, Pressure=%d\n", button, x, y, rt);
	}

	close( s_ts_fd );		//close touchscreen
	close( s_fb_fd );		//close frame buffer

	return 0 ;
}


/* ========================================================================= */
/**
 * @brief	- Display bitmap
 *
 * @par		usage:
 * @par		Note: \n
 *				1.
 *
 * @param	fileName;	image file name
 * @param	x;		x start position at LCD
 * @param	y;		y start position at LCD
 * @param	width;		width of image
 * @param	height;		height of image
 **/
/* ========================================================================= */
static int display_bitmap( char* fileName , int x, int y,  int width, int height)
{
	unsigned int		dataSize;
	struct stat			fileInfo;
	unsigned char*		dispData ;
	unsigned long		rawImageSize;

	FILE *sendFileFb = fopen( fileName,"rb" );
	if( sendFileFb == NULL){
		printf( " File Open Error %s\n",fileName );
		return -EFAULT ;
	}

	// Image header information read
	fseek( sendFileFb,0,SEEK_SET );
	fread( (void*)&bitFileHead, 1, sizeof(BITMAPFILEHEADER), sendFileFb );
	fread( (void*)&bitInfoHead, 1, sizeof(BITMAPINFOHEADER), sendFileFb );

	stat(fileName, &fileInfo);

	dataSize = bitInfoHead.biWidth * bitInfoHead.biHeight * BYTEPP;
	rawImageSize = bitInfoHead.biWidth*bitInfoHead.biHeight*bitInfoHead.biBitCount/8;
	if (dataSize < rawImageSize){
		dataSize = rawImageSize;
	}

	dispData = (unsigned char *)malloc( dataSize );

	memset( (void *)dispData, 0x00, dataSize );

	fread((void *)dispData, 1, rawImageSize, sendFileFb );

	// convert a bitmap file
	changeBitmapToVRAMData(
					bitInfoHead.biWidth,
					bitInfoHead.biHeight,
					rawImageSize,
					dispData );

	// memmap address copy
	// copies image to be displayed directly to the virtual frame buffer
	memcpy( _fb_addr, dispData, bitInfoHead.biWidth * bitInfoHead.biHeight * BYTEPP );

	free( dispData );
	fclose( sendFileFb );

	return 0 ;
}


/* ========================================================================= */
/**
 * @brief	- RGB:888 mode 1 converts
 *
 * @par		usage:
 * @par		Note: \n
 *				1. I acquire the RGB value of the conversion object
 *				2. Conversion process
 *
 * @param	width;	Width of the image
 * @param	height;	The height of the image
 * @param	size;	Conversion former size
 * @param	buf;	Buffer after a conversion former buffer/conversion
 * @retval	ret;	Size after conversion
 **/
/* ========================================================================= */
static int changeBitmapToVRAMData( unsigned int width, unsigned int height, unsigned int size, unsigned char *buf )
{
	register int		ii ;
	register int		jj ;
	register int		kk ;
	int					retSize = 0 ;
	unsigned char		*chgBuf ;
	unsigned int		chgBufSize;

	chgBufSize = SIZE_X * SIZE_Y * BYTEPP + 1;
	if (chgBufSize < size) {
		chgBufSize = size;
	}
	chgBuf =  (unsigned char *)malloc( chgBufSize );
	if( chgBuf==NULL ) {
		return -1 ;
	}
	memset( (void *)chgBuf, 0x00, chgBufSize );

	retSize = size ;
	jj = retSize;

	if( jj > 0 ) {
		// Top and bottom replacement
		for( ii=0, jj=retSize-3 ; ii<retSize ; ii+=3, jj-=3 ) {
			chgBuf[ii]		= (unsigned char)buf[jj];		// Blue
			chgBuf[ii+1]	= (unsigned char)buf[jj+1];		// Green
			chgBuf[ii+2]	= (unsigned char)buf[jj+2];		// Red
		}
		memcpy( buf, chgBuf, retSize );

		// Right and left replacement
		memset( (void *)chgBuf, 0x00, retSize ) ;
		for( kk=1, ii=0 ; kk <= (int)height ; kk++ ) {
			for( jj = (kk * (int)width * 3) - 3 ; ii < (kk * (int)width * 3) ; ii+=3, jj-=3 ) {
				chgBuf[ii]		= (unsigned char)buf[jj];
				chgBuf[ii+1]	= (unsigned char)buf[jj+1];
				chgBuf[ii+2]	= (unsigned char)buf[jj+2];
			}
		}
		memcpy( buf, chgBuf, retSize );
	}

	// RGB888 conversion
#if BITPP == 16
	unsigned char		tr;
	unsigned char		tg;
	unsigned char		tb;
	for( ii=0,jj=0 ; ii<(int)size ; ii+=3,jj+=2 ) {
			tb = buf[ii];
			tg = buf[ii+1];
			tr = buf[ii+2];
			chgBuf[jj+1] = ((tr>>3 )<<3) | (tg >>5) ;
			chgBuf[jj] = ((tg>>2)<<5) | (tb >>3 );
	}
#elif BITPP == 24
	for( ii=0,jj=0 ; ii<(int)size ; ii+=6,jj+=6 ) {
			chgBuf[jj+0] = buf[ii+1];		// G1 
			chgBuf[jj+1] = buf[ii+2];		// R1
			chgBuf[jj+2] = buf[ii+5];		// R2
			chgBuf[jj+3] = buf[ii+0];		// B1
			chgBuf[jj+4] = buf[ii+3];		// B2
			chgBuf[jj+5] = buf[ii+4];		// G2
	}
#elif BITPP == 32
	for( ii=0,jj=0 ; ii<(int)size ; ii+=3,jj+=4 ) {
			chgBuf[jj+0] = buf[ii+2];		// R1 
			chgBuf[jj+1] = 0;				// 0
			chgBuf[jj+2] = buf[ii+0];		// B1
			chgBuf[jj+3] = buf[ii+1];		// G1
	}
#endif
	memcpy( buf, chgBuf, jj);

	free( chgBuf );

	return retSize ;
}


