//-----------------------------------------------------------------------------
// linux/drivers/usb/misc/epson-13u11/test/13u11ts.c  
//
// This file demonstrates the touch screen for the S1D13U11 P00C.
// 
// Copyright(C) SEIKO EPSON CORPORATION 2010-2013. All rights reserved.
//
// This driver software is distributed as is, without any warranty of any kind,
// either express or implied as further specified in the GNU Public License. This
// software may be used and distributed according to the terms of the GNU Public
// License, version 2. See the file COPYING in the main directory of this archive
// for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//
//  Compile with:
//        gcc -s -Wall -Wstrict-prototypes 13u11ts.c -o 13u11ts
//
//-----------------------------------------------------------------------------
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <linux/fb.h>
#include <fcntl.h>
#include <math.h>
#include <errno.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <linux/input.h>

#include "../s1d13u11_ctl.h"

//#define		DEF_TS_DEVICE_NAME	"/dev/input/event2"
//#define		DEF_TS_DEVICE_NAME	"/dev/input/event3"
//#define		DEF_TS_DEVICE_NAME	"/dev/input/event4"
#define		DEF_TS_DEVICE_NAME	"/dev/input/event5"

#define __DEBUG
#ifdef __DEBUG
#define DPRINT(...)	printf(__VA_ARGS__)
#else
#define DPRINT(...)
#endif


//----------------------------------------------------------------------
// Grobal Variables
//----------------------------------------------------------------------
static int				s_ts_fd;


/* ========================================================================= */
/**
 * @brief	- S1D13U11 sample application
 *
 * @par		usage:
 * @par		Note: 
 *
 * @param	argc;	not used
 * @param	argv;	not used
 **/
/* ========================================================================= */
int main( int argc, char *argv[] )
{
	//struct s1d13u11_ctl pctl;
	//int ii = 0 ;
	
	//touchscreen variables
	struct input_event ts_event[8], *event, *last_event;	//touchscreen event
	int x=0, y=0, rt=0, button=0;		// x,y position, rt=pressure, button state
	int tslen=0, tsvalue=0;

	// open touchscreen
	s_ts_fd = open(DEF_TS_DEVICE_NAME,O_RDONLY);
	if (s_ts_fd < 0){
		printf( "Error Touchscreen Device![%d]\n",s_ts_fd );
		return -1;
	}


	//demonstrate touchscreen (touch lower right corner to exit)
	printf("Touch screen to report current x,y and pressure status.\n");
	printf("Touch the lower right corner to stop reporting.\n\n");
	while(x<3500 || y<3500){
		tslen = read(s_ts_fd, &ts_event, sizeof( ts_event));
		event = ts_event;
		last_event = (void *) ts_event + tslen;

		for (event = ts_event; event < last_event; event++){
			tsvalue = event->value;

			switch (event->type) {
			case EV_ABS:
				switch (event->code) {
				case ABS_X:
					x = tsvalue;
					break;
				case ABS_Y:
					y = tsvalue;
					break;
				case ABS_PRESSURE:
					rt = tsvalue;
					break;
				}

			case EV_KEY:
				if (tsvalue == 2)
					break;
				switch (event->code) {
				case BTN_TOUCH:
					button = tsvalue;
					break;
				}
			}
		}
		printf("Button=%d, X=%d, Y=%d, Pressure=%d\n", button, x, y, rt);
	}

	close( s_ts_fd );		//close touchscreen

	return 0 ;
}
